import asyncio
from unittest.mock import Mock, patch

import pytest
from pysnmp.hlapi.asyncio import (
    ObjectIdentity,
    ObjectType,
    Udp6TransportTarget,
    UdpTransportTarget,
)
from pysnmp.proto import errind
from pysnmp.proto.rfc1905 import EndOfMibView, NoSuchInstance, NoSuchObject

from zino.config.models import PollDevice
from zino.oid import OID
from zino.snmp import pysnmp_backend
from zino.snmp.base import (
    EndOfMibViewError,
    ErrorIndication,
    ErrorStatus,
    Identifier,
    MibNotFoundError,
    NoSuchInstanceError,
    NoSuchNameError,
    NoSuchObjectError,
)
from zino.snmp.pysnmp_backend import SNMP, init_backend


@pytest.fixture(scope="session")
def snmp_client(snmpsim, snmp_test_port):
    device = PollDevice(name="buick.lab.example.org", address="127.0.0.1", port=snmp_test_port)
    return SNMP(device)


@pytest.fixture(scope="session")
def ipv6_snmp_client(snmpsim, snmp_test_port) -> SNMP:
    device = PollDevice(name="buick.lab.example.org", address="::1", port=snmp_test_port)
    return SNMP(device)


@pytest.fixture()
def unreachable_snmp_client():
    mock_results = errind.RequestTimedOut(), None, None, []
    future = asyncio.Future()
    future.set_result(mock_results)
    timeout_mock = Mock(return_value=future)
    with patch.multiple("zino.snmp.pysnmp_backend", getCmd=timeout_mock, nextCmd=timeout_mock, bulkCmd=timeout_mock):
        device = PollDevice(name="nonexist", address="127.0.0.1", community="invalid", port=666)
        yield SNMP(device)


class TestSNMPRequestsResponseTypes:
    async def test_get(self, snmp_client):
        response = await snmp_client.get("SNMPv2-MIB", "sysUpTime", 0)
        assert isinstance(response.oid, OID)
        assert isinstance(response.value, int)

    async def test_get2_should_return_symbolic_identifiers(self, snmp_client):
        response = await snmp_client.get2(("IF-MIB", "ifName", 1), ("IF-MIB", "ifAlias", 1))
        assert len(list(response)) == 2
        assert any(identifier == Identifier("IF-MIB", "ifName", OID(".1")) for identifier, _ in response)
        assert any(identifier == Identifier("IF-MIB", "ifAlias", OID(".1")) for identifier, _ in response)

    async def test_when_mib_is_unkown_get2_should_raise_mibnotfounderror(self, snmp_client):
        with pytest.raises(MibNotFoundError):
            await snmp_client.get2(("FOOBAR-MIB", "ifName", 1), ("FOOBAR-MIB", "ifAlias", 1))

    async def test_getnext(self, snmp_client):
        response = await snmp_client.getnext("SNMPv2-MIB", "sysUpTime")
        assert isinstance(response.oid, OID)
        assert isinstance(response.value, int)

    async def test_getnext2_should_return_symbolic_identifiers(self, snmp_client):
        response = await snmp_client.getnext2(("IF-MIB", "ifName", "1"), ("IF-MIB", "ifAlias", "1"))
        assert len(list(response)) == 2
        assert any(identifier == Identifier("IF-MIB", "ifName", OID(".2")) for identifier, _ in response)
        assert any(identifier == Identifier("IF-MIB", "ifAlias", OID(".2")) for identifier, _ in response)

    async def test_walk(self, snmp_client):
        response = await snmp_client.walk("SNMPv2-MIB", "sysUpTime")
        assert response
        for mib_object in response:
            assert isinstance(mib_object.oid, OID)
            assert isinstance(mib_object.value, int)

    async def test_getbulk(self, snmp_client):
        response = await snmp_client.getbulk("SNMPv2-MIB", "sysUpTime")
        assert response
        for mib_object in response:
            assert isinstance(mib_object.oid, OID)
            assert isinstance(mib_object.value, int)

    async def test_getbulk2_should_have_expected_response(self, snmp_client):
        variables = ("ifIndex", "ifDescr", "ifAlias")
        response = await snmp_client.getbulk2(*(("IF-MIB", var) for var in variables))
        assert response
        for var_binds in response:
            assert len(var_binds) == len(variables)
            for ident, value in var_binds:
                assert ident.object in variables

    async def test_bulkwalk(self, snmp_client):
        response = await snmp_client.bulkwalk("SNMPv2-MIB", "sysUpTime")
        assert response
        for mib_object in response:
            assert isinstance(mib_object.oid, OID)
            assert isinstance(mib_object.value, int)

    async def test_sparsewalk_should_have_expected_response(self, snmp_client):
        variables = ("ifIndex", "ifDescr", "ifAlias")
        response = await snmp_client.sparsewalk(*(("IF-MIB", var) for var in variables))
        assert response
        for index, row in response.items():
            assert isinstance(index, OID)
            assert isinstance(row, dict)
            for var, val in row.items():
                assert var in variables

    async def test_get_sysobjectid_should_be_tuple_of_ints(self, snmp_client):
        response = await snmp_client.get("SNMPv2-MIB", "sysObjectID", 0)
        assert isinstance(response.oid, OID)
        assert isinstance(response.value, OID)
        assert all(isinstance(i, int) for i in response.value)

    async def test_get_named_value_should_return_symbolic_name(self, snmp_client):
        response = await snmp_client.getnext("SNMPv2-MIB", "snmpEnableAuthenTraps")
        assert response.value == "disabled"


class TestUnknownMibShouldRaiseException:
    async def test_get(self, snmp_client):
        with pytest.raises(MibNotFoundError):
            await snmp_client.get("NON-EXISTENT-MIB", "foo", 0)

    async def test_getnext(self, snmp_client):
        with pytest.raises(MibNotFoundError):
            await snmp_client.getnext("NON-EXISTENT-MIB", "foo")

    async def test_getnext2(self, snmp_client):
        with pytest.raises(MibNotFoundError):
            await snmp_client.getnext2(("NON-EXISTENT-MIB", "foo"))

    async def test_walk(self, snmp_client):
        with pytest.raises(MibNotFoundError):
            await snmp_client.walk("NON-EXISTENT-MIB", "foo")

    async def test_getbulk(self, snmp_client):
        with pytest.raises(MibNotFoundError):
            await snmp_client.getbulk("NON-EXISTENT-MIB", "foo")

    async def test_bulkwalk(self, snmp_client):
        with pytest.raises(MibNotFoundError):
            await snmp_client.bulkwalk("NON-EXISTENT-MIB", "foo")

    async def test_getbulk2(self, snmp_client):
        with pytest.raises(MibNotFoundError):
            await snmp_client.getbulk2(("NON-EXISTENT-MIB", "foo"))

    async def test_sparsewalk(self, snmp_client):
        with pytest.raises(MibNotFoundError):
            await snmp_client.sparsewalk(("NON-EXISTENT-MIB", "foo"))


class TestMibResolver:
    """Tests to ensure that various required MIBs can be resolved"""

    def test_sysuptime_should_be_resolved(self):
        object_type = SNMP._oid_to_object_type("SNMPv2-MIB", "sysUpTime")
        SNMP._resolve_object(object_type)
        assert object_type[0]

    def test_ifalias_should_be_resolved(self):
        object_type = SNMP._oid_to_object_type("IF-MIB", "ifAlias", "1")
        SNMP._resolve_object(object_type)
        assert object_type[0]

    def test_ipadentaddr_should_be_resolved(self):
        object_type = SNMP._oid_to_object_type("IP-MIB", "ipAdEntAddr")
        SNMP._resolve_object(object_type)
        assert object_type[0]

    def test_jnx_bgp_m2_peer_state_should_be_resolved(self):
        object_type = SNMP._oid_to_object_type("BGP4-V2-MIB-JUNIPER", "jnxBgpM2PeerState")
        SNMP._resolve_object(object_type)
        assert object_type[0]

    def test_c_bgp_peer2_state_should_be_resolved(self):
        object_type = SNMP._oid_to_object_type("CISCO-BGP4-MIB", "cbgpPeer2State")
        SNMP._resolve_object(object_type)
        assert object_type[0]

    def test_bgp_peer_state_should_be_resolved(self):
        object_type = SNMP._oid_to_object_type("BGP4-MIB", "bgpPeerState")
        SNMP._resolve_object(object_type)
        assert object_type[0]


class TestUnreachableDeviceShouldRaiseException:
    async def test_get(self, unreachable_snmp_client):
        with pytest.raises(TimeoutError):
            await unreachable_snmp_client.get("SNMPv2-MIB", "sysUpTime", 0)

    async def test_getnext(self, unreachable_snmp_client):
        with pytest.raises(TimeoutError):
            await unreachable_snmp_client.getnext("SNMPv2-MIB", "sysUpTime")

    async def test_getnext2(self, unreachable_snmp_client):
        with pytest.raises(TimeoutError):
            await unreachable_snmp_client.getnext2(("SNMPv2-MIB", "sysUpTime"))

    async def test_walk(self, unreachable_snmp_client):
        with pytest.raises(TimeoutError):
            await unreachable_snmp_client.walk("SNMPv2-MIB", "sysUpTime")

    async def test_getbulk(self, unreachable_snmp_client):
        with pytest.raises(TimeoutError):
            await unreachable_snmp_client.getbulk("SNMPv2-MIB", "sysUpTime")

    async def test_bulkwalk(self, unreachable_snmp_client):
        with pytest.raises(TimeoutError):
            await unreachable_snmp_client.bulkwalk("SNMPv2-MIB", "sysUpTime")

    async def test_getbulk2(self, unreachable_snmp_client):
        with pytest.raises(TimeoutError):
            await unreachable_snmp_client.getbulk2(("SNMPv2-MIB", "sysUpTime"))

    async def test_sparsewalk(self, unreachable_snmp_client):
        with pytest.raises(TimeoutError):
            await unreachable_snmp_client.sparsewalk(("SNMPv2-MIB", "sysUpTime"))


async def test_get_object_that_does_not_exist_should_raise_exception(snmp_client):
    with pytest.raises((NoSuchNameError, NoSuchInstanceError)):
        # NoSuchNameError is only relevant on SNMP v1, for v2c the error is NoSuchInstanceError
        await snmp_client.get("SNMPv2-MIB", "sysUpTime", 1)


class TestUdpTransportTarget:
    def test_when_device_address_is_ipv6_then_udp6_transport_should_be_returned(self, ipv6_snmp_client):
        assert isinstance(ipv6_snmp_client.udp_transport_target, Udp6TransportTarget)

    def test_when_device_address_is_ipv4_then_udp_transport_should_be_returned(self, snmp_client):
        assert isinstance(snmp_client.udp_transport_target, UdpTransportTarget)


class TestVarBindErrors:
    """
    Test class for verifying the handling of varbind errors in SNMP commands.

    This class contains tests that check if the correct exceptions are raised when
    varbind errors (NoSuchObject, NoSuchInstance, EndOfMibView) are encountered
    in the response to an SNMP command.
    """

    @pytest.mark.parametrize(
        "error, exception",
        [
            (NoSuchObject(""), NoSuchObjectError),
            (NoSuchInstance(""), NoSuchInstanceError),
            (EndOfMibView(""), EndOfMibViewError),
        ],
        ids=["NoSuchObject-NoSuchObjectError", "NoSuchInstance-NoSuchInstanceError", "EndOfMibView-EndOfMibViewError"],
    )
    async def test_get_should_raise_exception(self, error, exception, snmp_client, monkeypatch):
        query = ("SNMPv2-MIB", "sysDescr", 0)
        object_type = ObjectType(ObjectIdentity(*query), error)

        mock_results = None, None, None, [object_type]
        future = asyncio.Future()
        future.set_result(mock_results)
        get_mock = Mock(return_value=future)
        monkeypatch.setattr("zino.snmp.pysnmp_backend.getCmd", get_mock)

        snmp_client._resolve_object(object_type)

        with pytest.raises(exception):
            await snmp_client.get(*query)


class TestPDUErrors:
    """Tests for handling error flags in response PDUs."""

    async def test_when_snmpv1_variable_does_not_exist_it_should_raise_error(self, snmp_client):
        snmp_client.device.snmpversion = "v1"  # force SNMPv1
        with pytest.raises(NoSuchNameError):
            await snmp_client.get("SNMPv2-MIB", "sysUpTime", 999)

    def test_when_local_error_is_indicated_it_should_raise(self, snmp_client):
        with pytest.raises(ErrorIndication):
            snmp_client._raise_errors(error_indication="Something went wrong", error_status=0, error_index=0)

    def test_when_pdu_error_is_not_nosuchname_it_should_raise_generic_error(self, snmp_client):
        obj = snmp_client._oid_to_object_type("SNMPv2-MIB", "sysDescr")
        snmp_client._resolve_object(obj)
        error_status = 4  # readOnly
        error_index = 1
        with pytest.raises(ErrorStatus):
            snmp_client._raise_errors(None, error_status, error_index, obj)


class TestSubtreeIsSupported:
    async def test_when_agent_has_subtree_it_should_return_true(self, snmp_client):
        response = await snmp_client.subtree_is_supported("SNMPv2-MIB", "system")
        assert response

    async def test_when_agent_does_not_have_subtree_it_should_return_false(self, snmp_client):
        response = await snmp_client.subtree_is_supported("BFD-STD-MIB", "bfdSessTable")
        assert not response


class TestInitBackend:
    def test_it_should_initialize_an_snmp_engine_instance(self):
        init_backend()
        assert pysnmp_backend._local.snmp_engine is not None


def test_snmp_object_should_be_usable_as_context_manager(snmp_client):
    with snmp_client as context:
        assert context is snmp_client



#
# PySNMP MIB module CISCO-SMI (http://snmplabs.com/pysmi)
# ASN.1 source file://./CISCO-SMI.mib
# Produced by pysmi-1.1.10 at Tue Nov 28 11:02:57 2023
# On host simtve-ThinkPad-X1-Carbon-Gen-9 platform Linux version 5.15.0-84-generic by user simon
# Using Python version 3.9.18 (main, Aug 25 2023, 13:20:04) 
#
OctetString, Integer, ObjectIdentifier = mibBuilder.importSymbols("ASN1", "OctetString", "Integer", "ObjectIdentifier")
NamedValues, = mibBuilder.importSymbols("ASN1-ENUMERATION", "NamedValues")
ConstraintsIntersection, ValueSizeConstraint, ValueRangeConstraint, ConstraintsUnion, SingleValueConstraint = mibBuilder.importSymbols("ASN1-REFINEMENT", "ConstraintsIntersection", "ValueSizeConstraint", "ValueRangeConstraint", "ConstraintsUnion", "SingleValueConstraint")
NotificationGroup, ModuleCompliance = mibBuilder.importSymbols("SNMPv2-CONF", "NotificationGroup", "ModuleCompliance")
MibIdentifier, IpAddress, Unsigned32, iso, ObjectIdentity, TimeTicks, NotificationType, Integer32, MibScalar, MibTable, MibTableRow, MibTableColumn, Counter64, ModuleIdentity, Counter32, Gauge32, enterprises, Bits = mibBuilder.importSymbols("SNMPv2-SMI", "MibIdentifier", "IpAddress", "Unsigned32", "iso", "ObjectIdentity", "TimeTicks", "NotificationType", "Integer32", "MibScalar", "MibTable", "MibTableRow", "MibTableColumn", "Counter64", "ModuleIdentity", "Counter32", "Gauge32", "enterprises", "Bits")
DisplayString, TextualConvention = mibBuilder.importSymbols("SNMPv2-TC", "DisplayString", "TextualConvention")
cisco = ModuleIdentity((1, 3, 6, 1, 4, 1, 9))
cisco.setRevisions(('2016-01-15 00:00', '2012-08-29 00:00', '2009-02-03 00:00', '2002-03-21 00:00', '2001-05-22 00:00', '2000-11-01 22:46', '2000-01-11 00:00', '1997-04-09 00:00', '1995-05-16 00:00', '1994-04-26 20:00',))

if getattr(mibBuilder, 'version', (0, 0, 0)) > (4, 4, 0):
    if mibBuilder.loadTexts: cisco.setRevisionsDescriptions(('Added ciscoLDAP under cisco', 'Added ciscoSMB under otherEnterprises', 'Added ciscoSB under otherEnterprises', 'Added ciscoPKI for PKI policy and extension OIDs', 'Added transport protocol domains.', 'Added ciscoDomains to define new transports. Also added ciscoCpeCIB, which will contain managed objects that contribute to the CPE Configuration Information Base (CIB).', 'Added ciscoPolicy, ciscoPolicyAuto, ciscoPIB, and ciscoPibToMib.', 'Added ciscoPartnerProducts to generate sysObjectID for partner platforms', 'New oid assignments for Cisco REPEATER MIB and others.', 'Initial version of this MIB module.',))
if mibBuilder.loadTexts: cisco.setLastUpdated('201601150000Z')
if mibBuilder.loadTexts: cisco.setOrganization('Cisco Systems, Inc.')
if mibBuilder.loadTexts: cisco.setContactInfo('Cisco Systems Customer Service Postal: 170 West Tasman Drive San Jose, CA 95134 USA Tel: +1 800 553-NETS E-mail: cs-snmp@cisco.com')
if mibBuilder.loadTexts: cisco.setDescription('The Structure of Management Information for the Cisco enterprise.')
ciscoProducts = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 1))
if mibBuilder.loadTexts: ciscoProducts.setStatus('current')
if mibBuilder.loadTexts: ciscoProducts.setDescription('ciscoProducts is the root OBJECT IDENTIFIER from which sysObjectID values are assigned. Actual values are defined in CISCO-PRODUCTS-MIB.')
local = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 2))
if mibBuilder.loadTexts: local.setStatus('current')
if mibBuilder.loadTexts: local.setDescription('Subtree beneath which pre-10.2 MIBS were built.')
temporary = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 3))
if mibBuilder.loadTexts: temporary.setStatus('current')
if mibBuilder.loadTexts: temporary.setDescription('Subtree beneath which pre-10.2 experiments were placed.')
pakmon = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 4))
if mibBuilder.loadTexts: pakmon.setStatus('current')
if mibBuilder.loadTexts: pakmon.setDescription('reserved for pakmon')
workgroup = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 5))
if mibBuilder.loadTexts: workgroup.setStatus('current')
if mibBuilder.loadTexts: workgroup.setDescription('subtree reserved for use by the Workgroup Business Unit')
otherEnterprises = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 6))
if mibBuilder.loadTexts: otherEnterprises.setStatus('current')
if mibBuilder.loadTexts: otherEnterprises.setDescription('otherEnterprises provides a root object identifier from which mibs produced by other companies may be placed. mibs produced by other enterprises are typicially implemented with the object identifiers as defined in the mib, but if the mib is deemed to be uncontrolled, we may reroot the mib at this subtree in order to have a controlled version.')
ciscoSB = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 6, 1))
if mibBuilder.loadTexts: ciscoSB.setStatus('current')
if mibBuilder.loadTexts: ciscoSB.setDescription('ciscoSB provides root Object Identifier for Management Information Base for products of Cisco Small Business. This includes products rebranded from linksys aquisition. MIB numbers under this root are managed and controlled by ciscosb_mib@cisco.com.')
ciscoSMB = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 6, 2))
if mibBuilder.loadTexts: ciscoSMB.setStatus('current')
if mibBuilder.loadTexts: ciscoSMB.setDescription('ciscoSMB provides root Object Identifier for Management Information Base for products of Cisco built for Small and Medium Business market.The MIB numbers under this root are managed and controlled by ciscosmb_mib@cisco.com')
ciscoAgentCapability = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 7))
if mibBuilder.loadTexts: ciscoAgentCapability.setStatus('current')
if mibBuilder.loadTexts: ciscoAgentCapability.setDescription('ciscoAgentCapability provides a root object identifier from which AGENT-CAPABILITIES values may be assigned.')
ciscoConfig = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 8))
if mibBuilder.loadTexts: ciscoConfig.setStatus('current')
if mibBuilder.loadTexts: ciscoConfig.setDescription('ciscoConfig is the main subtree for configuration mibs.')
ciscoMgmt = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 9))
if mibBuilder.loadTexts: ciscoMgmt.setStatus('current')
if mibBuilder.loadTexts: ciscoMgmt.setDescription('ciscoMgmt is the main subtree for new mib development.')
ciscoExperiment = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 10))
if mibBuilder.loadTexts: ciscoExperiment.setStatus('current')
if mibBuilder.loadTexts: ciscoExperiment.setDescription('ciscoExperiment provides a root object identifier from which experimental mibs may be temporarily based. mibs are typicially based here if they fall in one of two categories 1) are IETF work-in-process mibs which have not been assigned a permanent object identifier by the IANA. 2) are cisco work-in-process which has not been assigned a permanent object identifier by the cisco assigned number authority, typicially because the mib is not ready for deployment. NOTE WELL: support for mibs in the ciscoExperiment subtree will be deleted when a permanent object identifier assignment is made.')
ciscoAdmin = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11))
if mibBuilder.loadTexts: ciscoAdmin.setStatus('current')
if mibBuilder.loadTexts: ciscoAdmin.setDescription('ciscoAdmin is reserved for administratively assigned OBJECT IDENTIFIERS, i.e. those not associated with MIB objects')
ciscoModules = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 12))
if mibBuilder.loadTexts: ciscoModules.setStatus('current')
if mibBuilder.loadTexts: ciscoModules.setDescription('ciscoModules provides a root object identifier from which MODULE-IDENTITY values may be assigned.')
lightstream = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 13))
if mibBuilder.loadTexts: lightstream.setStatus('current')
if mibBuilder.loadTexts: lightstream.setDescription('subtree reserved for use by Lightstream')
ciscoworks = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 14))
if mibBuilder.loadTexts: ciscoworks.setStatus('current')
if mibBuilder.loadTexts: ciscoworks.setDescription('ciscoworks provides a root object identifier beneath which mibs applicable to the CiscoWorks family of network management products are defined.')
newport = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 15))
if mibBuilder.loadTexts: newport.setStatus('current')
if mibBuilder.loadTexts: newport.setDescription('subtree reserved for use by the former Newport Systems Solutions, now a portion of the Access Business Unit.')
ciscoPartnerProducts = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 16))
if mibBuilder.loadTexts: ciscoPartnerProducts.setStatus('current')
if mibBuilder.loadTexts: ciscoPartnerProducts.setDescription('ciscoPartnerProducts is the root OBJECT IDENTIFIER from which partner sysObjectID values may be assigned. Such sysObjectID values are composed of the ciscoPartnerProducts prefix, followed by a single identifier that is unique for each partner, followed by the value of sysObjectID of the Cisco product from which partner product is derived. Note that the chassisPartner MIB object defines the value of the identifier assigned to each partner.')
ciscoPolicy = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 17))
if mibBuilder.loadTexts: ciscoPolicy.setStatus('current')
if mibBuilder.loadTexts: ciscoPolicy.setDescription('ciscoPolicy is the root of the Cisco-assigned OID subtree for use with Policy Management.')
ciscoPIB = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 17, 2))
if mibBuilder.loadTexts: ciscoPIB.setStatus('current')
if mibBuilder.loadTexts: ciscoPIB.setDescription('ciscoPIB is the root of the Cisco-assigned OID subtree for assignment to PIB (Policy Information Base) modules.')
ciscoPolicyAuto = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 18))
if mibBuilder.loadTexts: ciscoPolicyAuto.setStatus('current')
if mibBuilder.loadTexts: ciscoPolicyAuto.setDescription('ciscoPolicyAuto is the root of the Cisco-assigned OID subtree for OIDs which are automatically assigned for use in Policy Management.')
ciscoPibToMib = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 18, 2))
if mibBuilder.loadTexts: ciscoPibToMib.setStatus('current')
if mibBuilder.loadTexts: ciscoPibToMib.setDescription("ciscoPibToMib is the root of the Cisco-assigned OID subtree for MIBs which are algorithmically generated/translated from Cisco PIBs with OIDs assigned under the ciscoPIB subtree. These generated MIBs allow management entities (other the current Policy Server) to read the downloaded policy. By convention, for PIB 'ciscoPIB.x', the generated MIB shall have the name 'ciscoPibToMib.x'.")
ciscoDomains = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19))
if mibBuilder.loadTexts: ciscoDomains.setStatus('current')
if mibBuilder.loadTexts: ciscoDomains.setDescription('ciscoDomains provides a root object identifier from which different transport mapping values may be assigned.')
ciscoCIB = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 20))
if mibBuilder.loadTexts: ciscoCIB.setStatus('current')
if mibBuilder.loadTexts: ciscoCIB.setDescription('ciscoCIB is the root of the Cisco-assigned OID subtree for assignment to MIB modules describing managed objects that part of the CPE automatic configuration framework.')
ciscoCibMmiGroup = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 20, 1))
if mibBuilder.loadTexts: ciscoCibMmiGroup.setStatus('current')
if mibBuilder.loadTexts: ciscoCibMmiGroup.setDescription('ciscoCibMmiGroup is the root of the Cisco-assigned OID subtree for assignment to MIB modules describing managed objects supporting the Modem Management Interface (MMI), the interface that facilitates CPE automatic configuration.')
ciscoCibProvGroup = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 20, 2))
if mibBuilder.loadTexts: ciscoCibProvGroup.setStatus('current')
if mibBuilder.loadTexts: ciscoCibProvGroup.setDescription('ciscoCibStoreGroup is the root of the Cisco-assigned OID subtree for assignment to MIB modules describing managed objects contributing to the Configuration Information Base (CIB).')
ciscoPKI = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 21))
if mibBuilder.loadTexts: ciscoPKI.setStatus('current')
if mibBuilder.loadTexts: ciscoPKI.setDescription('ciscoPKI is the root of cisco-assigned OID subtree for PKI Certificate Policies and Certificate Extensions.')
ciscoLDAP = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 22))
if mibBuilder.loadTexts: ciscoLDAP.setStatus('current')
if mibBuilder.loadTexts: ciscoLDAP.setDescription('ciscoLDAP is the root of the Cisco-assigned OID subtree for assignment to LDAP (Lightweight Directory Access Protocol) modules.')
ciscoProxy = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 1))
if mibBuilder.loadTexts: ciscoProxy.setStatus('current')
if mibBuilder.loadTexts: ciscoProxy.setDescription('ciscoProxy OBJECT IDENTIFIERS are used to uniquely name party mib records created to proxy for SNMPv1.')
ciscoPartyProxy = MibIdentifier((1, 3, 6, 1, 4, 1, 9, 11, 1, 1))
ciscoContextProxy = MibIdentifier((1, 3, 6, 1, 4, 1, 9, 11, 1, 2))
ciscoRptrGroupObjectID = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 2))
if mibBuilder.loadTexts: ciscoRptrGroupObjectID.setStatus('current')
if mibBuilder.loadTexts: ciscoRptrGroupObjectID.setDescription('ciscoRptrGroupObjectID OBJECT IDENTIFIERS are used to uniquely identify groups of repeater ports for use by the SNMP-REPEATER-MIB (RFC 1516) rptrGroupObjectID object.')
ciscoUnknownRptrGroup = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 2, 1))
if mibBuilder.loadTexts: ciscoUnknownRptrGroup.setStatus('current')
if mibBuilder.loadTexts: ciscoUnknownRptrGroup.setDescription('The identity of an unknown repeater port group.')
cisco2505RptrGroup = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 2, 2))
if mibBuilder.loadTexts: cisco2505RptrGroup.setStatus('current')
if mibBuilder.loadTexts: cisco2505RptrGroup.setDescription('The authoritative identity of the Cisco 2505 repeater port group.')
cisco2507RptrGroup = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 2, 3))
if mibBuilder.loadTexts: cisco2507RptrGroup.setStatus('current')
if mibBuilder.loadTexts: cisco2507RptrGroup.setDescription('The authoritative identity of the Cisco 2507 repeater port group.')
cisco2516RptrGroup = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 2, 4))
if mibBuilder.loadTexts: cisco2516RptrGroup.setStatus('current')
if mibBuilder.loadTexts: cisco2516RptrGroup.setDescription('The authoritative identity of the Cisco 2516 repeater port group.')
ciscoWsx5020RptrGroup = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 2, 5))
if mibBuilder.loadTexts: ciscoWsx5020RptrGroup.setStatus('current')
if mibBuilder.loadTexts: ciscoWsx5020RptrGroup.setDescription('The authoritative identity of the wsx5020 repeater port group.')
ciscoChipSets = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 3))
if mibBuilder.loadTexts: ciscoChipSets.setStatus('current')
if mibBuilder.loadTexts: ciscoChipSets.setDescription('Numerous media-specific MIBS have an object, defined as an OBJECT IDENTIFIER, which is the identity of the chipset realizing the interface. Cisco-specific chipsets have their OBJECT IDENTIFIERS assigned under this subtree.')
ciscoChipSetSaint1 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 3, 1))
if mibBuilder.loadTexts: ciscoChipSetSaint1.setStatus('current')
if mibBuilder.loadTexts: ciscoChipSetSaint1.setDescription('The identity of the Rev 1 SAINT ethernet chipset manufactured for cisco by LSI Logic.')
ciscoChipSetSaint2 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 3, 2))
if mibBuilder.loadTexts: ciscoChipSetSaint2.setStatus('current')
if mibBuilder.loadTexts: ciscoChipSetSaint2.setDescription('The identity of the Rev 2 SAINT ethernet chipset manufactured for cisco by LSI Logic.')
ciscoChipSetSaint3 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 3, 3))
if mibBuilder.loadTexts: ciscoChipSetSaint3.setStatus('current')
if mibBuilder.loadTexts: ciscoChipSetSaint3.setDescription('The identity of the Rev 3 SAINT ethernet chipset manufactured for cisco by Plessey.')
ciscoChipSetSaint4 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 11, 3, 4))
if mibBuilder.loadTexts: ciscoChipSetSaint4.setStatus('current')
if mibBuilder.loadTexts: ciscoChipSetSaint4.setDescription('The identity of the Rev 4 SAINT ethernet chipset manufactured for cisco by Mitsubishi.')
ciscoTDomains = MibIdentifier((1, 3, 6, 1, 4, 1, 9, 19, 99999))
ciscoTDomainUdpIpv4 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 1))
if mibBuilder.loadTexts: ciscoTDomainUdpIpv4.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainUdpIpv4.setDescription('The UDP over IPv4 transport domain. The corresponding transport address is of type CiscoTAddressIPv4.')
ciscoTDomainUdpIpv6 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 2))
if mibBuilder.loadTexts: ciscoTDomainUdpIpv6.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainUdpIpv6.setDescription('The UDP over IPv6 transport domain. The corresponding transport address is of type CiscoTAddressIPv6 for global IPv6 addresses and CiscoTAddressIPv6s for scoped IPv6 addresses.')
ciscoTDomainTcpIpv4 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 3))
if mibBuilder.loadTexts: ciscoTDomainTcpIpv4.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainTcpIpv4.setDescription('The TCP over IPv4 transport domain. The corresponding transport address is of type CiscoTAddressIPv4.')
ciscoTDomainTcpIpv6 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 4))
if mibBuilder.loadTexts: ciscoTDomainTcpIpv6.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainTcpIpv6.setDescription('The TCP over IPv6 transport domain. The corresponding transport address is of type CiscoTAddressIPv6 for global IPv6 addresses and CiscoTAddressIPv6s for scoped IPv6 addresses.')
ciscoTDomainLocal = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 5))
if mibBuilder.loadTexts: ciscoTDomainLocal.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainLocal.setDescription('The Posix Local IPC transport domain. The corresponding transport address is of type CiscoTAddressLocal. The Posix Local IPC transport domain incorporates the well known UNIX domain sockets.')
ciscoTDomainClns = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 6))
if mibBuilder.loadTexts: ciscoTDomainClns.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainClns.setDescription('The CLNS transport domain. The corresponding transport address is of type CiscoTAddressOSI.')
ciscoTDomainCons = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 7))
if mibBuilder.loadTexts: ciscoTDomainCons.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainCons.setDescription('The CONS transport domain. The corresponding transport address is of type CiscoTAddressOSI.')
ciscoTDomainDdp = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 8))
if mibBuilder.loadTexts: ciscoTDomainDdp.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainDdp.setDescription('The DDP transport domain. The corresponding transport address is of type CiscoTAddressNBP.')
ciscoTDomainIpx = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 9))
if mibBuilder.loadTexts: ciscoTDomainIpx.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainIpx.setDescription('The IPX transport domain. The corresponding transport address is of type CiscoTAddressIPX.')
ciscoTDomainSctpIpv4 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 10))
if mibBuilder.loadTexts: ciscoTDomainSctpIpv4.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainSctpIpv4.setDescription('The SCTP over IPv4 transport domain. The corresponding transport address is of type CiscoTAddressIPv4.')
if mibBuilder.loadTexts: ciscoTDomainSctpIpv4.setReference('RFC 2960 - Stream Control Transmission Protocol. R. Stewart, Q. Xie, K. Morneault, C. Sharp, H. Schwarzbauer, T. Taylor, I. Rytina, M. Kalla, L. Zhang, V. Paxson. October 2000.')
ciscoTDomainSctpIpv6 = ObjectIdentity((1, 3, 6, 1, 4, 1, 9, 19, 99999, 11))
if mibBuilder.loadTexts: ciscoTDomainSctpIpv6.setStatus('current')
if mibBuilder.loadTexts: ciscoTDomainSctpIpv6.setDescription('The SCTP over IPv6 transport domain. The corresponding transport address is of type CiscoTAddressIPv6 for global IPv6 addresses and CiscoTAddressIPv6s for scoped IPv6 addresses.')
if mibBuilder.loadTexts: ciscoTDomainSctpIpv6.setReference('RFC 2960 - Stream Control Transmission Protocol. R. Stewart, Q. Xie, K. Morneault, C. Sharp, H. Schwarzbauer, T. Taylor, I. Rytina, M. Kalla, L. Zhang, V. Paxson. October 2000.')
mibBuilder.exportSymbols("CISCO-SMI", ciscoRptrGroupObjectID=ciscoRptrGroupObjectID, ciscoDomains=ciscoDomains, ciscoChipSets=ciscoChipSets, ciscoPKI=ciscoPKI, ciscoPartnerProducts=ciscoPartnerProducts, ciscoAdmin=ciscoAdmin, ciscoPolicyAuto=ciscoPolicyAuto, ciscoTDomains=ciscoTDomains, ciscoChipSetSaint4=ciscoChipSetSaint4, ciscoTDomainUdpIpv6=ciscoTDomainUdpIpv6, ciscoTDomainCons=ciscoTDomainCons, cisco=cisco, ciscoSB=ciscoSB, pakmon=pakmon, ciscoAgentCapability=ciscoAgentCapability, newport=newport, cisco2507RptrGroup=cisco2507RptrGroup, ciscoworks=ciscoworks, ciscoTDomainUdpIpv4=ciscoTDomainUdpIpv4, ciscoTDomainTcpIpv4=ciscoTDomainTcpIpv4, ciscoCIB=ciscoCIB, ciscoTDomainDdp=ciscoTDomainDdp, ciscoCibProvGroup=ciscoCibProvGroup, cisco2516RptrGroup=cisco2516RptrGroup, ciscoTDomainLocal=ciscoTDomainLocal, ciscoPartyProxy=ciscoPartyProxy, ciscoWsx5020RptrGroup=ciscoWsx5020RptrGroup, ciscoTDomainTcpIpv6=ciscoTDomainTcpIpv6, ciscoTDomainClns=ciscoTDomainClns, ciscoTDomainSctpIpv6=ciscoTDomainSctpIpv6, ciscoPibToMib=ciscoPibToMib, temporary=temporary, otherEnterprises=otherEnterprises, ciscoTDomainIpx=ciscoTDomainIpx, ciscoProxy=ciscoProxy, ciscoTDomainSctpIpv4=ciscoTDomainSctpIpv4, PYSNMP_MODULE_ID=cisco, ciscoCibMmiGroup=ciscoCibMmiGroup, ciscoPolicy=ciscoPolicy, workgroup=workgroup, ciscoChipSetSaint1=ciscoChipSetSaint1, ciscoLDAP=ciscoLDAP, ciscoChipSetSaint3=ciscoChipSetSaint3, local=local, ciscoUnknownRptrGroup=ciscoUnknownRptrGroup, ciscoSMB=ciscoSMB, ciscoExperiment=ciscoExperiment, ciscoConfig=ciscoConfig, ciscoModules=ciscoModules, ciscoPIB=ciscoPIB, cisco2505RptrGroup=cisco2505RptrGroup, lightstream=lightstream, ciscoContextProxy=ciscoContextProxy, ciscoMgmt=ciscoMgmt, ciscoProducts=ciscoProducts, ciscoChipSetSaint2=ciscoChipSetSaint2)



#
# PySNMP MIB module JUNIPER-SMI (http://snmplabs.com/pysmi)
# ASN.1 source file://./JUNIPER-SMI.mib
# Produced by pysmi-1.1.10 at Thu Jul  6 12:36:59 2023
# On host simtve-ThinkPad-X1-Carbon-Gen-9 platform Linux version 5.14.0-1058-oem by user simon
# Using Python version 3.9.16 (main, Dec  7 2022, 01:11:51) 
#
OctetString, ObjectIdentifier, Integer = mibBuilder.importSymbols("ASN1", "OctetString", "ObjectIdentifier", "Integer")
NamedValues, = mibBuilder.importSymbols("ASN1-ENUMERATION", "NamedValues")
ValueSizeConstraint, ConstraintsIntersection, ValueRangeConstraint, SingleValueConstraint, ConstraintsUnion = mibBuilder.importSymbols("ASN1-REFINEMENT", "ValueSizeConstraint", "ConstraintsIntersection", "ValueRangeConstraint", "SingleValueConstraint", "ConstraintsUnion")
ModuleCompliance, NotificationGroup = mibBuilder.importSymbols("SNMPv2-CONF", "ModuleCompliance", "NotificationGroup")
Integer32, Unsigned32, Gauge32, ObjectIdentity, ModuleIdentity, Counter64, iso, IpAddress, MibIdentifier, Bits, Counter32, MibScalar, MibTable, MibTableRow, MibTableColumn, enterprises, TimeTicks, NotificationType = mibBuilder.importSymbols("SNMPv2-SMI", "Integer32", "Unsigned32", "Gauge32", "ObjectIdentity", "ModuleIdentity", "Counter64", "iso", "IpAddress", "MibIdentifier", "Bits", "Counter32", "MibScalar", "MibTable", "MibTableRow", "MibTableColumn", "enterprises", "TimeTicks", "NotificationType")
DisplayString, TextualConvention = mibBuilder.importSymbols("SNMPv2-TC", "DisplayString", "TextualConvention")
juniperMIB = ModuleIdentity((1, 3, 6, 1, 4, 1, 2636))
juniperMIB.setRevisions(('2010-07-09 00:00', '2009-10-29 00:00', '2010-06-18 00:00', '2003-04-17 01:00', '2005-08-17 01:00', '2006-12-14 01:00', '2007-01-01 00:00', '2007-10-09 00:00', '2009-12-31 00:00', '2010-07-14 00:00', '2011-01-26 00:00', '2012-02-10 00:00', '2012-08-01 00:00', '2012-11-01 00:00', '2012-12-07 00:00', '2013-01-25 00:00', '2013-11-26 00:00', '2014-10-09 00:00', '2014-10-29 00:00', '2015-11-19 00:00', '2016-05-31 00:00', '2017-03-01 00:00',))
if mibBuilder.loadTexts: juniperMIB.setLastUpdated('201703010000Z')
if mibBuilder.loadTexts: juniperMIB.setOrganization('Juniper Networks, Inc.')
jnxProducts = ObjectIdentity((1, 3, 6, 1, 4, 1, 2636, 1))
if mibBuilder.loadTexts: jnxProducts.setStatus('current')
jnxMediaFlow = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 1, 2))
jnxJunosSpace = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 1, 3))
jnxReservedProducts3 = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 1, 4))
jnxReservedProducts4 = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 1, 5))
jnxReservedProducts5 = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 1, 6))
jnxSDKApplicationsRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 1, 7))
jnxJAB = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 1, 8))
jnxStrm = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 1, 9))
jnxServices = ObjectIdentity((1, 3, 6, 1, 4, 1, 2636, 2))
if mibBuilder.loadTexts: jnxServices.setStatus('current')
jnxMibs = ObjectIdentity((1, 3, 6, 1, 4, 1, 2636, 3))
if mibBuilder.loadTexts: jnxMibs.setStatus('current')
jnxJsMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 39))
jnxExMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 40))
jnxWxMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 41))
jnxDcfMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 42))
jnxReservedMibs5 = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 43))
jnxPfeMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 44))
jnxBfdMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 45))
jnxXstpMibs = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 46))
jnxUtilMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 47))
jnxl2aldMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 48))
jnxL2tpMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 49))
jnxRpmMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 50))
jnxUserAAAMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 51))
jnxIpSecMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 52))
jnxL2cpMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 53))
jnxPwTdmMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 54))
jnxPwTCMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 55))
jnxOtnMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 56))
jnxPsuMIBRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 58))
jnxSvcsMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 59))
jnxDomMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 60))
jnxJdhcpMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 61))
jnxJdhcpv6MibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 62))
jnxLicenseMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 63))
jnxSubscriberMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 64))
jnxMagMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 65))
jnxMobileGatewayMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 66))
jnxPppoeMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 67))
jnxPppMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 68))
jnxJVAEMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 69))
jnxIfOtnMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 70))
jnxOpticsMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 71))
jnxAlarmExtMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 72))
jnxoptIfMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 73))
jnxFruMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 74))
jnxTimingNotfnsMIBRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 75))
jnxSnmpSetMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 76))
jnxTwampMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 77))
jnxVmonMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 79))
jnxSGMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 80))
jnxFabricMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 81))
jnxSRDMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 82))
jnxOamMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 83))
jnxTunnelStatsMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 84))
jnxURLFMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 3, 85))
jnxTraps = ObjectIdentity((1, 3, 6, 1, 4, 1, 2636, 4))
if mibBuilder.loadTexts: jnxTraps.setStatus('current')
jnxChassisTraps = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 1))
jnxChassisOKTraps = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 2))
jnxRmonTraps = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 3))
jnxLdpTraps = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 4))
jnxCmNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 5))
jnxSonetNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 6))
jnxPMonNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 7))
jnxCollectorNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 8))
jnxPingNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 9))
jnxSpNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 10))
jnxDfcNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 11))
jnxSyslogNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 12))
jnxEventNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 13))
jnxVccpNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 14))
jnxOtnNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 15))
jnxSAIDPNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 16))
jnxCosNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 17))
jnxDomNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 18))
jnxFabricChassisTraps = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 19))
jnxFabricChassisOKTraps = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 20))
jnxIfOtnNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 21))
jnxOpticsNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 22))
jnxFruTraps = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 23))
jnxSnmpSetTraps = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 24))
jnxDomLaneNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 25))
jnxTwampNotificationPrefix = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 27))
jnxIplcNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 28))
jnxIlaNotifications = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 4, 29))
jnxExperiment = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 5))
jnxNsm = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 6))
jnxCA = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 7))
jnxAAA = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 8))
jnxAdvancedInsightMgr = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 9))
jnxBxMibRoot = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 10))
jnxAgentCapability = MibIdentifier((1, 3, 6, 1, 4, 1, 2636, 11))
mibBuilder.exportSymbols("JUNIPER-SMI", jnxTunnelStatsMibRoot=jnxTunnelStatsMibRoot, jnxStrm=jnxStrm, jnxSnmpSetMibRoot=jnxSnmpSetMibRoot, jnxSpNotifications=jnxSpNotifications, jnxTwampNotificationPrefix=jnxTwampNotificationPrefix, jnxPppoeMibRoot=jnxPppoeMibRoot, jnxOtnNotifications=jnxOtnNotifications, jnxFabricMibRoot=jnxFabricMibRoot, jnxOpticsNotifications=jnxOpticsNotifications, jnxLicenseMibRoot=jnxLicenseMibRoot, jnxDcfMibRoot=jnxDcfMibRoot, jnxTimingNotfnsMIBRoot=jnxTimingNotfnsMIBRoot, jnxMibs=jnxMibs, jnxL2cpMibRoot=jnxL2cpMibRoot, jnxIfOtnMibRoot=jnxIfOtnMibRoot, jnxFabricChassisOKTraps=jnxFabricChassisOKTraps, jnxCA=jnxCA, jnxFruTraps=jnxFruTraps, jnxJdhcpMibRoot=jnxJdhcpMibRoot, PYSNMP_MODULE_ID=juniperMIB, jnxChassisOKTraps=jnxChassisOKTraps, jnxPwTdmMibRoot=jnxPwTdmMibRoot, jnxoptIfMibRoot=jnxoptIfMibRoot, jnxJAB=jnxJAB, jnxUtilMibRoot=jnxUtilMibRoot, jnxXstpMibs=jnxXstpMibs, jnxPMonNotifications=jnxPMonNotifications, jnxWxMibRoot=jnxWxMibRoot, jnxLdpTraps=jnxLdpTraps, jnxL2tpMibRoot=jnxL2tpMibRoot, jnxProducts=jnxProducts, jnxDomNotifications=jnxDomNotifications, jnxDomLaneNotifications=jnxDomLaneNotifications, jnxJVAEMibRoot=jnxJVAEMibRoot, jnxCollectorNotifications=jnxCollectorNotifications, jnxPfeMibRoot=jnxPfeMibRoot, jnxDomMibRoot=jnxDomMibRoot, jnxNsm=jnxNsm, jnxIlaNotifications=jnxIlaNotifications, jnxSDKApplicationsRoot=jnxSDKApplicationsRoot, jnxEventNotifications=jnxEventNotifications, jnxSvcsMibRoot=jnxSvcsMibRoot, jnxVmonMibRoot=jnxVmonMibRoot, jnxAAA=jnxAAA, jnxOamMibRoot=jnxOamMibRoot, jnxMagMibRoot=jnxMagMibRoot, jnxTwampMibRoot=jnxTwampMibRoot, jnxSonetNotifications=jnxSonetNotifications, jnxSRDMibRoot=jnxSRDMibRoot, jnxPppMibRoot=jnxPppMibRoot, jnxVccpNotifications=jnxVccpNotifications, jnxPsuMIBRoot=jnxPsuMIBRoot, jnxSubscriberMibRoot=jnxSubscriberMibRoot, juniperMIB=juniperMIB, jnxIplcNotifications=jnxIplcNotifications, jnxRmonTraps=jnxRmonTraps, jnxFabricChassisTraps=jnxFabricChassisTraps, jnxSnmpSetTraps=jnxSnmpSetTraps, jnxReservedMibs5=jnxReservedMibs5, jnxSGMibRoot=jnxSGMibRoot, jnxDfcNotifications=jnxDfcNotifications, jnxReservedProducts5=jnxReservedProducts5, jnxPwTCMibRoot=jnxPwTCMibRoot, jnxReservedProducts4=jnxReservedProducts4, jnxURLFMibRoot=jnxURLFMibRoot, jnxIfOtnNotifications=jnxIfOtnNotifications, jnxExMibRoot=jnxExMibRoot, jnxAdvancedInsightMgr=jnxAdvancedInsightMgr, jnxExperiment=jnxExperiment, jnxMediaFlow=jnxMediaFlow, jnxServices=jnxServices, jnxOtnMibRoot=jnxOtnMibRoot, jnxCosNotifications=jnxCosNotifications, jnxFruMibRoot=jnxFruMibRoot, jnxTraps=jnxTraps, jnxCmNotifications=jnxCmNotifications, jnxPingNotifications=jnxPingNotifications, jnxMobileGatewayMibRoot=jnxMobileGatewayMibRoot, jnxAgentCapability=jnxAgentCapability, jnxChassisTraps=jnxChassisTraps, jnxReservedProducts3=jnxReservedProducts3, jnxJdhcpv6MibRoot=jnxJdhcpv6MibRoot, jnxJunosSpace=jnxJunosSpace, jnxl2aldMibRoot=jnxl2aldMibRoot, jnxJsMibRoot=jnxJsMibRoot, jnxIpSecMibRoot=jnxIpSecMibRoot, jnxBfdMibRoot=jnxBfdMibRoot, jnxRpmMibRoot=jnxRpmMibRoot, jnxUserAAAMibRoot=jnxUserAAAMibRoot, jnxOpticsMibRoot=jnxOpticsMibRoot, jnxSyslogNotifications=jnxSyslogNotifications, jnxAlarmExtMibRoot=jnxAlarmExtMibRoot, jnxBxMibRoot=jnxBxMibRoot, jnxSAIDPNotifications=jnxSAIDPNotifications)



import pytest
from pydantic import ValidationError

from zino.config import InvalidConfigurationError, read_configuration


class TestReadConfiguration:
    def test_returns_config_defined_in_file(self, zino_conf, polldevs_conf_with_no_routers):
        config = read_configuration(zino_conf)
        assert config
        assert config.polling.file == str(polldevs_conf_with_no_routers)

    def test_pollfile_argument_overrides_pollfile_defined_in_file(self, zino_conf, polldevs_conf_with_single_router):
        config = read_configuration(config_file_name=zino_conf, poll_file_name=str(polldevs_conf_with_single_router))
        assert config
        assert config.polling.file == str(polldevs_conf_with_single_router)

    def test_raises_error_on_file_not_found(self, tmp_path):
        with pytest.raises(FileNotFoundError):
            read_configuration(tmp_path / "non-existent-config.toml")

    def test_raises_error_on_invalid_toml_file(self, invalid_zino_conf):
        with pytest.raises(InvalidConfigurationError):
            read_configuration(invalid_zino_conf)

    def test_raises_error_on_invalid_config_values(self, invalid_values_zino_conf):
        with pytest.raises(ValidationError) as excinfo:
            read_configuration(invalid_values_zino_conf)

        assert "archiving.old_events_dir" in str(excinfo)

    def tests_raises_error_on_misspelled_key(self, extra_keys_zino_conf):
        with pytest.raises(ValidationError) as excinfo:
            read_configuration(extra_keys_zino_conf)

        assert "Extra inputs are not permitted" in str(excinfo)

    def test_raises_error_on_pollfile_not_found(self, zino_conf_with_non_existent_pollfile):
        with pytest.raises(ValidationError) as excinfo:
            read_configuration(zino_conf_with_non_existent_pollfile)

        assert "polling.file" in str(excinfo.value)
        assert "non-existent-pollfile.cf doesn't exist or isn't readable" in str(excinfo.value)
